﻿//	Settings class
//	--	
//	Read and write settings to an XML .config file. Does not use 
//	ConfigurationSettings.AppSettings since it's not supported on 
//	.NET Compact Framework. 
//
//	Uses same schema as app.config file. Example:
//
//		<configuration>
//			<appSettings>
//				<app key="Name" value="Live Oak" />
//				<app key="LogEvents" value="True" />
//			</appSettings>
//		</configuration>	
//
//	Default settings file name is the same as app.config, 
//	appends .config to the end of the assembly name. Example:
//
//		<appname.exe>.config

using System.Collections;
using System.Xml;
using System.IO;


// Read and write .config settings.
public class Settings
{


    public sealed class ControlChars
    {
        public const char Back = '\b';
        public const char Cr = '\r';
        public const string CrLf = "\r\n";
        public const char FormFeed = '\f';
        public const char Lf = '\n';
        public const string NewLine = "\r\n";
        public const char NullChar = '\0';
        public const char Quote = '"';
        public const char Tab = '\t';
        public const char VerticalTab = '\v';
    }

    // internal fields
    private Hashtable _list = new Hashtable();
    private string _filePath = "";
    private bool _autoWrite = true;

    private string[,] _defaultValues;
    // Specifies if the settings file is updated whenever a value 
    // is set. If false, you need to call Write to update the 
    // underlying settings file.
    public bool AutoWrite
    {
        get { return _autoWrite; }

        set { _autoWrite = value; }
    }

    // Full path to settings file.
    public string FilePath
    {
        get { return _filePath; }

        set { _filePath = value; }
    }

    // Default constructor. 
    public Settings()
    {
        // get full path to file
        _filePath = GetFilePath();

        // populate list with settings from file
        Read();
    }


    // Constructor. Pass in an array of default values.
    // Sample usage that passes in default values:
    //
    //		string[,] values = 
    //		{
    //			{"Name", "Live Oak"},
    //			{"LogEvents", "True"}
    //		}
    //
    //		Settings settings = new Settings(values);
    public Settings(string[,] defaultValues)
    {
        // store default values, use later when populate list
        _defaultValues = defaultValues;

        // get full path to file
        //_filePath = GetFilePath() + "\\" + System.Windows.Forms.Application.ProductName() + ".xml";
        
        _filePath = GetFilePath();

        // populate list with settings from file
        Read();
    }

    // Set setting value. Update	underlying file	if AutoUpdate is true.
    public void SetValue(string key, object value)
    {
        // update internal list
        this._list[key] = value;

        // update settings file
        if (_autoWrite)
        {
            this.Write();
        }
    }

    // Return specified settings	as string.
    public string GetString(string key)
    {
        object result = this._list[key];
        if (result == null)
        {
            return "";
        }
        else
        {
            return result.ToString();
        }
    }

    // Return specified settings as integer.
    public int GetInt(string key)
    {
        string result = GetString(key);
        if (string.IsNullOrEmpty(result))
        {
            return 0;
        }
        else
        {
            return System.Convert.ToInt32(result);
        }
    }

    // Return specified settings as boolean.
    public bool GetBool(string key)
    {
        string result = GetString(key);
        if (string.IsNullOrEmpty(result))
        {
            return false;
        }
        else
        {
            return System.Convert.ToBoolean(result);
        }
    }

    // Read settings file.
    public void Read()
    {
        // first remove all items from list
        _list.Clear();

        // next, populate list with default values
        int i = 0;
        for (i = 0; i <= (_defaultValues.GetLength(0)) - 1; i++)
        {
            _list[_defaultValues[i, 0]] = _defaultValues[i, 1];
        }

        // last, populate list with items from file
        // open settings file
        if (File.Exists(_filePath))
        {
            XmlTextReader reader = new XmlTextReader(_filePath);

            // go through file and read the xml file and 
            // populate internal list with 'add' elements
            while (reader.Read())
            {
                if (reader.NodeType == XmlNodeType.Element & reader.Name == "app")
                {
                    _list[reader.GetAttribute("key")] = reader.GetAttribute("value");
                }
            }

            reader.Close();
        }
    }


    // Write settings to the .config file.
    public void Write()
    {
        // header elements
        StreamWriter writer = File.CreateText(_filePath);
        writer.WriteLine("<configuration>");
        writer.WriteLine(ControlChars.Tab + "<appSettings>");

        // go through internal list and create 'add' element for each item
        IDictionaryEnumerator enumerator = _list.GetEnumerator();
        while (enumerator.MoveNext())
        {
            writer.WriteLine(ControlChars.Tab + ControlChars.Tab + "<app key=\"{0}\" value=\"{1}\" />", enumerator.Key.ToString(), enumerator.Value);
        }

        // footer elements
        writer.WriteLine(ControlChars.Tab + "</appSettings>");
        writer.WriteLine("</configuration>");

        writer.Close();
    }

    // Overwrite settings with default values.
    public void RestoreDefaults()
    {
        try
        {
            // easiest way is to delete the file and
            // repopulate internal list with defaults
            File.Delete(FilePath);
            Read();
        }
        catch (System.Exception ex)
        {
            System.Diagnostics.Debug.WriteLine(ex.Message);
        }
    }

    // Return full path to settings file. Appends .config to the assembly name.
    private string GetFilePath()
    {
        string myPath = string.Empty;
        // myPath = System.IO.Path.GetDirectoryName(System.Windows.Forms.Application.ExecutablePath); // PC only
        myPath = System.Reflection.Assembly.GetExecutingAssembly().GetName().CodeBase + ".config";
        return myPath;
    }

}